@@ -7,9 +7,11 @@ import IPython
 import lz4
 import inspect
 import funcsigs
+import operator
 import matplotlib.pyplot as plt
 plt.ion()
 from colorama import Fore as F
+import pprint
 
 try:
     import pandas as pd
@@ -19,6 +21,7 @@ except ImportError:
 
 try:
     from . import client_helper
+    from . import help_content
     from . import version
     from . import file_streamer
     from . import repr_to_str_decorator
@@ -28,6 +31,7 @@ try:
 # super handy and we'll keep it even though it hurts coverage score.
 except (ImportError,ValueError):
     import client_helper
+    import help_content
     import version
     import file_streamer
     import repr_to_str_decorator
@@ -43,6 +47,9 @@ class WorkbenchShell(object):
         # Workbench CLI version
         self.version = version.__version__
 
+        # Workbench CLI Help
+        self.help = help_content.WorkbenchShellHelp()
+
         # Grab server arguments
         self.server_info = client_helper.grab_server_args()
 
@@ -106,6 +113,9 @@ class WorkbenchShell(object):
                 print '\n%s  %s%s %sLocked and Loaded...%s\n' % \
                       (self.beer, F.MAGENTA, md5[:6], F.YELLOW, F.RESET)
 
+                # Add tags to the sample
+                self.workbench.add_tags(md5, tags)
+
                 # Store information about the sample into the sesssion
                 basename = os.path.basename(path)
                 self.session.filename = basename
@@ -114,6 +124,23 @@ class WorkbenchShell(object):
                 self.ipshell.push({'md5': self.session.md5})
                 self.ipshell.push({'short_md5': self.session.short_md5})
 
+        # Dump out tag information
+        self.tag_info()
+
+    def tag_info(self):
+        tags = self.workbench.get_all_tags()
+        if not tags:
+            return
+        tag_df = pd.DataFrame(tags)
+        tag_df = self.flatten_tags(tag_df)
+        del tag_df['md5']
+        del tag_df['tags']
+        tag_freq = tag_df.sum().to_dict()
+        tag_freq = sorted(tag_freq.iteritems(), key=operator.itemgetter(1), reverse=True)
+        print '\n%sSamples in Database%s' % (F.MAGENTA, F.RESET)
+        for (tag, count) in tag_freq:
+            print '  %s%s: %s%s%s' % (F.GREEN, tag, F.BLUE, count, F.RESET)
+
     def pull_df(self, md5):
         """Wrapper for the Workbench get_dataframe method
             Args:
@@ -128,20 +155,22 @@ class WorkbenchShell(object):
         except zerorpc.exceptions.RemoteError as e:
             return repr_to_str_decorator.r_to_s(self._data_not_found)(e)
 
-    def search(self, tags='all'):
+    def flatten_tags(self, my_df):
+        """Flatten(vectorize) the tags column in the dataframe"""
+        tags_df = my_df['tags'].str.join(sep='-').str.get_dummies(sep='-')
+        my_df['tags'] = [', '.join(tag_list) for tag_list in my_df['tags']]
+        return my_df.join(tags_df)
+
+    def search(self, tags=None):
         """Wrapper for the Workbench search method
             Args:
-                tags: a list of tags to search for ['bad','aptz13']
+                tags: a single tag 'pcap' or a list of tags to search for ['bad','aptz13']
             Returns:
-                The list of md5s for all matching samples
+                A sample_set that contains the md5s for all matching samples
         """
-
-        # Fixme: This needs to be improved to handle arbitrary predicates (MongoDB predicates)
-        if tags == 'all':
-            return [item['md5'] for item in self.workbench.list_samples()]
-        elif isinstance(tags, str):
+        if isinstance(tags, str):
             tags = [tags]
-        return [item['md5'] for item in self.workbench.list_samples({'tags': {'$in': tags}})]
+        return self.workbench.generate_sample_set(tags)
 
     def versions(self):
         """Announce Versions of CLI and Server
@@ -158,8 +187,9 @@ class WorkbenchShell(object):
         # Announce versions
         self.versions()
 
-        # Help
+        # Help and Sample/Tag info
         print '\n%s' % self.workbench.help('cli')
+        self.tag_info()
 
         # Now that we have the Workbench connection spun up, we register some stuff
         # with the embedded IPython interpreter and than spin it up
@@ -230,6 +260,10 @@ class WorkbenchShell(object):
         elif not md5:
             md5 = self.session.md5
 
+        # Is the md5 a sample_set?
+        if self.workbench.is_sample_set(md5):
+            return self.workbench.set_work_request(worker, md5)
+
         # Make the work_request with worker and md5 args
         try:
             return self.workbench.work_request(worker, md5)
@@ -262,6 +296,8 @@ class WorkbenchShell(object):
             'help': self._help,
             'load_sample': self.load_sample,
             'pull_df': self.pull_df,
+            'flatten_tags': self.flatten_tags,
+            'tag_info': self.tag_info,
             'search': self.search,
             'reconnect': lambda info=self.server_info: self._connect(info),
             'version': self.versions,
@@ -292,30 +328,13 @@ class WorkbenchShell(object):
                 self.workbench.store_info(info, name, 'command')
 
         # Register help information
-        self.workbench.store_info({'help': self._help_cli()}, 'cli', 'help')
-        self.workbench.store_info({'help': self._help_cli_basic()}, 'cli_basic', 'help')
-
-    def _help_cli(self):
-        """ Help on Workbench CLI """
-        help = '%sWelcome to Workbench CLI Help:%s' % (F.YELLOW, F.RESET)
-        help += '\n\t%s> help cli_basic %s for getting started help' % (F.GREEN, F.BLUE)
-        help += '\n\t%s> help workers %s for help on available workers' % (F.GREEN, F.BLUE)
-        help += '\n\t%s> help commands %s for help on workbench commands' % (F.GREEN, F.BLUE)
-        help += '\n\t%s> help topic %s where topic can be a help, command or worker' % (F.GREEN, F.BLUE)
-        help += '\n\n%sNote: cli commands are transformed into python calls' % (F.YELLOW)
-        help += '\n\t%s> help cli_basic --> help("cli_basic")%s' % (F.GREEN, F.RESET)
-        return help
-
-    def _help_cli_basic(self):
-        """ Help for Workbench CLI Basics """
-        help =  '%sWorkbench: Getting started...' % (F.YELLOW)
-        help += '\n%sLoad in a sample:'  % (F.GREEN)
-        help += '\n\t%s> load_sample /path/to/file' % (F.BLUE)
-        help += '\n\n%sNotice the prompt now shows the md5 of the sample...'% (F.YELLOW)
-        help += '\n%sRun workers on the sample:'  % (F.GREEN)
-        help += '\n\t%s> view or meta or whatever... %s' % (F.BLUE, F.RESET)
-        return help
-        
+        self.workbench.store_info({'help': self.help.help_cli()}, 'cli', 'help')
+        self.workbench.store_info({'help': self.help.help_cli_basic()}, 'cli_basic', 'help')
+        self.workbench.store_info({'help': self.help.help_cli_search()}, 'search', 'help')
+        self.workbench.store_info({'help': self.help.help_dataframe()}, 'dataframe', 'help')
+        self.workbench.store_info({'help': self.help.help_dataframe_memory()}, 'dataframe_memory', 'help')
+        self.workbench.store_info({'help': self.help.help_dataframe_pe()}, 'dataframe_pe', 'help')
+ 
     def _help(self, topic=None):
         """Help wrapper for Workbench CLI"""
         if not topic:
